@@ -1,5 +1,4 @@
-use comfy_table::presets::UTF8_FULL;
-use comfy_table::*;
+use comfy_table::{presets::UTF8_FULL, *};
 
 fn main() {
     let mut table = Table::new();
