@@ -1,9 +1,6 @@
+use comfy_table::{modifiers::*, presets::*, *};
 use pretty_assertions::assert_eq;
 
-use comfy_table::modifiers::*;
-use comfy_table::presets::*;
-use comfy_table::*;
-
 fn get_preset_table() -> Table {
     let mut table = Table::new();
     table
