@@ -953,6 +953,46 @@ async fn head() {
     .assert_stdout("foo\nbar\nbaz\n")
     .run()
     .await;
+
+  // negative line count: -n -1 (all but last 1 line)
+  TestBuilder::new()
+    .command("head -n -1")
+    .stdin("line1\nline2\nline3\nline4\nline5\n")
+    .assert_stdout("line1\nline2\nline3\nline4\n")
+    .run()
+    .await;
+
+  // negative line count: -n -2 (all but last 2 lines)
+  TestBuilder::new()
+    .command("head -n -2")
+    .stdin("line1\nline2\nline3\nline4\nline5\n")
+    .assert_stdout("line1\nline2\nline3\n")
+    .run()
+    .await;
+
+  // negative line count with --lines
+  TestBuilder::new()
+    .command("head --lines=-3")
+    .stdin("line1\nline2\nline3\nline4\nline5\n")
+    .assert_stdout("line1\nline2\n")
+    .run()
+    .await;
+
+  // negative line count with file
+  TestBuilder::new()
+    .command("head -n -1 file")
+    .file("file", "line1\nline2\nline3\nline4\nline5\n")
+    .assert_stdout("line1\nline2\nline3\nline4\n")
+    .run()
+    .await;
+
+  // negative line count where skip >= total lines (empty output)
+  TestBuilder::new()
+    .command("head -n -10")
+    .stdin("line1\nline2\nline3\n")
+    .assert_stdout("")
+    .run()
+    .await;
 }
 
 // Basic integration tests as there are unit tests in the commands
@@ -1067,6 +1107,28 @@ async fn rm() {
     .assert_exit_code(1)
     .run()
     .await;
+
+  // rm -f should ignore non-existent files
+  TestBuilder::new()
+    .command("rm -f nonexistent.txt")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // rm -rf should ignore non-existent directories
+  TestBuilder::new()
+    .command("rm -rf nonexistent_dir")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // rm -rf with glob pattern that matches nothing should succeed
+  // (when failglob is disabled, the pattern is passed literally to rm)
+  TestBuilder::new()
+    .command("shopt -u failglob && rm -rf *.nonexistent")
+    .assert_exit_code(0)
+    .run()
+    .await;
 }
 
 // Basic integration tests as there are unit tests in the commands
@@ -1350,6 +1412,7 @@ async fn glob_basic() {
     .run()
     .await;
 
+  // ** matches recursively (globstar on by default)
   TestBuilder::new()
     .directory("sub_dir/sub")
     .file("sub_dir/sub/1.txt", "1\n")
@@ -1385,7 +1448,7 @@ async fn glob_basic() {
     .file("test.txt", "test\n")
     .file("test2.txt", "test2\n")
     .command("cat *.ts")
-    .assert_stderr("glob: no matches found '$TEMP_DIR/*.ts'\n")
+    .assert_stderr("glob: no matches found '$TEMP_DIR/*.ts' (run `shopt -u failglob` to pass unmatched glob patterns literally)\n")
     .assert_exit_code(1)
     .run()
     .await;
@@ -1405,7 +1468,7 @@ async fn glob_basic() {
     .file("test.txt", "test\n")
     .file("test2.txt", "test2\n")
     .command("cat *.ts || echo 2")
-    .assert_stderr("glob: no matches found '$TEMP_DIR/*.ts'\n")
+    .assert_stderr("glob: no matches found '$TEMP_DIR/*.ts' (run `shopt -u failglob` to pass unmatched glob patterns literally)\n")
     .assert_stdout("2\n")
     .assert_exit_code(0)
     .run()
@@ -1677,3 +1740,278 @@ async fn sigpipe_from_pipeline() {
     .run()
     .await;
 }
+
+#[tokio::test]
+async fn shopt() {
+  // query all options (default: failglob on, globstar on, nullglob off)
+  TestBuilder::new()
+    .command("shopt")
+    .assert_stdout("failglob\ton\nglobstar\ton\nnullglob\toff\n")
+    .run()
+    .await;
+
+  // query specific option
+  TestBuilder::new()
+    .command("shopt nullglob")
+    .assert_stdout("nullglob\toff\n")
+    .assert_exit_code(1) // returns 1 when option is off
+    .run()
+    .await;
+
+  TestBuilder::new()
+    .command("shopt failglob")
+    .assert_stdout("failglob\ton\n")
+    .assert_exit_code(0) // returns 0 when option is on
+    .run()
+    .await;
+
+  // enable option
+  TestBuilder::new()
+    .command("shopt -s nullglob && shopt nullglob")
+    .assert_stdout("nullglob\ton\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // disable option
+  TestBuilder::new()
+    .command("shopt -u failglob && shopt failglob")
+    .assert_stdout("failglob\toff\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // invalid option name
+  TestBuilder::new()
+    .command("shopt -s invalidopt")
+    .assert_stderr("shopt: invalidopt: invalid shell option name\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // -s without option name
+  TestBuilder::new()
+    .command("shopt -s")
+    .assert_stderr("shopt: option name required\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // cannot set and unset simultaneously
+  TestBuilder::new()
+    .command("shopt -s -u nullglob")
+    .assert_stderr("shopt: cannot set and unset options simultaneously\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // multiple options
+  TestBuilder::new()
+    .command("shopt -s nullglob && shopt -u failglob && shopt")
+    .assert_stdout("failglob\toff\nglobstar\ton\nnullglob\ton\n")
+    .run()
+    .await;
+
+  // query globstar option (on by default)
+  TestBuilder::new()
+    .command("shopt globstar")
+    .assert_stdout("globstar\ton\n")
+    .assert_exit_code(0) // returns 0 when option is on
+    .run()
+    .await;
+
+  // disable globstar
+  TestBuilder::new()
+    .command("shopt -u globstar && shopt globstar")
+    .assert_stdout("globstar\toff\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+}
+
+#[tokio::test]
+async fn shopt_nullglob() {
+  // default behavior (failglob on): unmatched glob causes error
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("echo *.nonexistent")
+    .assert_stderr("glob: no matches found '$TEMP_DIR/*.nonexistent' (run `shopt -u failglob` to pass unmatched glob patterns literally)\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // with nullglob: unmatched glob expands to nothing
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("shopt -u failglob && shopt -s nullglob && echo *.nonexistent")
+    .assert_stdout("\n") // echo with no args outputs newline
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // nullglob with other args: unmatched glob removed, other args kept
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("shopt -u failglob && shopt -s nullglob && echo hello *.nonexistent world")
+    .assert_stdout("hello world\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // nullglob: matched glob still works normally
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("shopt -u failglob && shopt -s nullglob && cat *.txt")
+    .assert_stdout("test\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+}
+
+#[tokio::test]
+async fn shopt_failglob() {
+  // failglob on (default): unmatched glob causes error
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("echo *.nonexistent")
+    .assert_stderr("glob: no matches found '$TEMP_DIR/*.nonexistent' (run `shopt -u failglob` to pass unmatched glob patterns literally)\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // failglob off: unmatched glob passed through literally (bash default)
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("shopt -u failglob && echo *.nonexistent")
+    .assert_stdout("*.nonexistent\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // failglob takes precedence over nullglob
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("shopt -s nullglob && shopt -s failglob && echo *.nonexistent")
+    .assert_stderr("glob: no matches found '$TEMP_DIR/*.nonexistent' (run `shopt -u failglob` to pass unmatched glob patterns literally)\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // failglob off, nullglob off: literal pattern (bash default)
+  TestBuilder::new()
+    .file("test.txt", "test\n")
+    .command("shopt -u failglob && shopt -u nullglob && echo *.nonexistent")
+    .assert_stdout("*.nonexistent\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+}
+
+#[tokio::test]
+async fn shopt_globstar() {
+  // globstar on by default: ** matches zero or more directories
+  // use cat to verify all files are matched (content order verifies glob worked)
+  TestBuilder::new()
+    .directory("sub/deep")
+    .file("a.txt", "a\n")
+    .file("sub/b.txt", "b\n")
+    .file("sub/deep/c.txt", "c\n")
+    .command("cat **/*.txt")
+    .assert_stdout("a\nb\nc\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // single * still behaves normally even with globstar enabled
+  TestBuilder::new()
+    .directory("sub")
+    .file("a.txt", "a\n")
+    .file("sub/b.txt", "b\n")
+    .command("echo *.txt")
+    .assert_stdout("a.txt\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // disabling globstar: ** behaves like * (single-segment match)
+  TestBuilder::new()
+    .directory("sub/deep")
+    .file("a.txt", "a\n")
+    .file("sub/b.txt", "b\n")
+    .file("sub/deep/c.txt", "c\n")
+    .command("shopt -u globstar && cat **/*.txt")
+    .assert_stdout("b\n") // only matches one level deep (sub/b.txt)
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // re-enabling globstar restores recursive behavior
+  TestBuilder::new()
+    .directory("sub/deep")
+    .file("a.txt", "a\n")
+    .file("sub/b.txt", "b\n")
+    .file("sub/deep/c.txt", "c\n")
+    .command("shopt -u globstar && shopt -s globstar && cat **/*.txt")
+    .assert_stdout("a\nb\nc\n")
+    .assert_exit_code(0)
+    .run()
+    .await;
+}
+
+#[tokio::test]
+async fn pipefail_option() {
+  // Without pipefail: exit code is from last command (0)
+  TestBuilder::new()
+    .command("(exit 1) | true")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // With pipefail: exit code is rightmost non-zero (1)
+  TestBuilder::new()
+    .command("set -o pipefail && (exit 1) | true")
+    .assert_exit_code(1)
+    .run()
+    .await;
+
+  // Multiple failures - should return rightmost non-zero
+  TestBuilder::new()
+    .command("set -o pipefail && (exit 2) | (exit 3) | true")
+    .assert_exit_code(3)
+    .run()
+    .await;
+
+  // All succeed - should return 0
+  TestBuilder::new()
+    .command("set -o pipefail && true | true | true")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // Disable pipefail with +o
+  TestBuilder::new()
+    .command("set -o pipefail && set +o pipefail && (exit 1) | true")
+    .assert_exit_code(0)
+    .run()
+    .await;
+
+  // invalid option name
+  TestBuilder::new()
+    .command("set -o invalidopt")
+    .assert_stderr("set: unknown option: invalidopt\n")
+    .assert_exit_code(1)
+    .run()
+    .await;
+}
+
+#[tokio::test]
+#[cfg(unix)]
+async fn pipefail_with_sigpipe() {
+  // With pipefail and SIGPIPE: should return 141 (128 + 13)
+  TestBuilder::new()
+    .command("set -o pipefail && yes | head -n 1")
+    .assert_stdout("y\n")
+    .assert_exit_code(141)
+    .run()
+    .await;
+}
