@@ -17,8 +17,8 @@ pub(crate) const GOOGLE_CLOUD_QUOTA_PROJECT_VAR: &str = "GOOGLE_CLOUD_QUOTA_PROJ
 /// Token Exchange OAuth Grant Type
 pub(crate) const TOKEN_EXCHANGE_GRANT_TYPE: &str =
     "urn:ietf:params:oauth:grant-type:token-exchange";
+#[cfg(feature = "idtoken")]
 /// JWT Bearer OAuth Grant Type
-#[cfg(google_cloud_unstable_id_token)]
 pub(crate) const JWT_BEARER_GRANT_TYPE: &str = "urn:ietf:params:oauth:grant-type:jwt-bearer";
 pub(crate) const STS_TOKEN_URL: &str = "https://sts.googleapis.com/v1/token";
 pub(crate) const OAUTH2_TOKEN_SERVER_URL: &str = "https://oauth2.googleapis.com/token";
@@ -29,6 +29,8 @@ pub(crate) const JWT_TOKEN_TYPE: &str = "urn:ietf:params:oauth:token-type:jwt";
 /// SAML2 Token OAuth Token Type
 pub(crate) const SAML2_TOKEN_TYPE: &str = "urn:ietf:params:oauth:token-type:saml2";
 
-pub(crate) const RETRY_EXHAUSTED_ERROR: &str = "All retry attempts to fetch the token were exhausted. Subsequent calls with this credential will also fail.";
+pub(crate) const RETRY_EXHAUSTED_ERROR: &str = "Fetching the token resulted in transient errors. Subsequent calls with this credential might succeed.";
 pub(crate) const TOKEN_FETCH_FAILED_ERROR: &str =
     "Request to fetch the token failed. Subsequent calls with this credential will also fail.";
+
+pub(crate) const TRUST_BOUNDARY_HEADER: &str = "x-goog-allowed-locations";
