@@ -12,21 +12,25 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+//! Types and functions to work with Google Cloud authentication [Credentials].
+//!
+//! [Credentials]: https://cloud.google.com/docs/authentication#credentials
+
 use crate::build_errors::Error as BuilderError;
 use crate::constants::GOOGLE_CLOUD_QUOTA_PROJECT_VAR;
 use crate::errors::{self, CredentialsError};
+use crate::token::Token;
 use crate::{BuildResult, Result};
 use http::{Extensions, HeaderMap};
 use serde_json::Value;
 use std::future::Future;
 use std::sync::Arc;
 use std::sync::atomic::{AtomicU64, Ordering};
-
 pub mod anonymous;
 pub mod api_key_credentials;
 pub mod external_account;
 pub(crate) mod external_account_sources;
-#[cfg(google_cloud_unstable_id_token)]
+#[cfg(feature = "idtoken")]
 pub mod idtoken;
 pub mod impersonated;
 pub(crate) mod internal;
@@ -111,7 +115,7 @@ pub struct Credentials {
     // will not do.
     //
     // They also need to derive `Clone`, as the
-    // `gax::http_client::ReqwestClient`s which hold them derive `Clone`. So a
+    // `google_cloud_gax::http_client::ReqwestClient`s which hold them derive `Clone`. So a
     // `Box` will not do.
     inner: Arc<dyn dynamic::CredentialsProvider>,
 }
@@ -137,6 +141,98 @@ impl Credentials {
     }
 }
 
+/// An implementation of [crate::credentials::CredentialsProvider] that can also
+/// provide direct access to the underlying access token.
+///
+/// This struct is returned by the `build_access_token_credentials()` method on
+/// the various credential builders. It can be used to obtain an access token
+/// directly via the `access_token()` method, or it can be converted into a `Credentials`
+/// object to be used with the Google Cloud client libraries.
+#[derive(Clone, Debug)]
+pub struct AccessTokenCredentials {
+    // We use an `Arc` to hold the inner implementation.
+    //
+    // AccessTokenCredentials may be shared across threads (`Send + Sync`), so an `Rc`
+    // will not do.
+    //
+    // They also need to derive `Clone`, as the
+    // `google_cloud_gax::http_client::ReqwestClient`s which hold them derive `Clone`. So a
+    // `Box` will not do.
+    inner: Arc<dyn dynamic::AccessTokenCredentialsProvider>,
+}
+
+impl<T> std::convert::From<T> for AccessTokenCredentials
+where
+    T: crate::credentials::AccessTokenCredentialsProvider + Send + Sync + 'static,
+{
+    fn from(value: T) -> Self {
+        Self {
+            inner: Arc::new(value),
+        }
+    }
+}
+
+impl AccessTokenCredentials {
+    pub async fn access_token(&self) -> Result<AccessToken> {
+        self.inner.access_token().await
+    }
+}
+
+/// Makes [AccessTokenCredentials] compatible with clients that expect
+/// a [Credentials] and/or a [CredentialsProvider].
+impl CredentialsProvider for AccessTokenCredentials {
+    async fn headers(&self, extensions: Extensions) -> Result<CacheableResource<HeaderMap>> {
+        self.inner.headers(extensions).await
+    }
+
+    async fn universe_domain(&self) -> Option<String> {
+        self.inner.universe_domain().await
+    }
+}
+
+/// Represents an OAuth 2.0 access token.
+#[derive(Clone)]
+pub struct AccessToken {
+    /// The access token string.
+    pub token: String,
+}
+
+impl std::fmt::Debug for AccessToken {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("AccessToken")
+            .field("token", &"[censored]")
+            .finish()
+    }
+}
+
+impl std::convert::From<CacheableResource<Token>> for Result<AccessToken> {
+    fn from(token: CacheableResource<Token>) -> Self {
+        match token {
+            CacheableResource::New { data, .. } => Ok(data.into()),
+            CacheableResource::NotModified => Err(errors::CredentialsError::from_msg(
+                false,
+                "Expecting token to be present",
+            )),
+        }
+    }
+}
+
+impl std::convert::From<Token> for AccessToken {
+    fn from(token: Token) -> Self {
+        Self { token: token.token }
+    }
+}
+
+/// A trait for credential types that can provide direct access to an access token.
+///
+/// This trait is primarily intended for interoperability with other libraries that
+/// require a raw access token, or for calling Google Cloud APIs that are not yet
+/// supported by the SDK.
+pub trait AccessTokenCredentialsProvider: CredentialsProvider + std::fmt::Debug {
+    /// Asynchronously retrieves an access token.
+    fn access_token(&self) -> impl Future<Output = Result<AccessToken>> + Send;
+}
+
 /// Represents a [Credentials] used to obtain auth request headers.
 ///
 /// In general, [Credentials][credentials-link] are "digital object that
@@ -256,6 +352,24 @@ pub(crate) mod dynamic {
             T::universe_domain(self).await
         }
     }
+
+    /// A dyn-compatible, crate-private version of `AccessTokenCredentialsProvider`.
+    #[async_trait::async_trait]
+    pub trait AccessTokenCredentialsProvider:
+        CredentialsProvider + Send + Sync + std::fmt::Debug
+    {
+        async fn access_token(&self) -> Result<super::AccessToken>;
+    }
+
+    #[async_trait::async_trait]
+    impl<T> AccessTokenCredentialsProvider for T
+    where
+        T: super::AccessTokenCredentialsProvider + Send + Sync,
+    {
+        async fn access_token(&self) -> Result<super::AccessToken> {
+            T::access_token(self).await
+        }
+    }
 }
 
 /// A builder for constructing [`Credentials`] instances.
@@ -397,13 +511,43 @@ impl Builder {
     ///
     /// # Errors
     ///
-    /// Returns a [CredentialsError] if a unsupported credential type is provided
-    /// or if the JSON value is either malformed
-    /// or missing required fields. For more information, on how to generate
-    /// json, consult the relevant section in the [application-default credentials] guide.
+    /// Returns a [CredentialsError] if an unsupported credential type is provided
+    /// or if the JSON value is either malformed or missing required fields.
+    ///
+    /// For more information, on how to generate the JSON for a credential,
+    /// consult the relevant section in the [application-default credentials] guide.
     ///
     /// [application-default credentials]: https://cloud.google.com/docs/authentication/application-default-credentials
     pub fn build(self) -> BuildResult<Credentials> {
+        Ok(self.build_access_token_credentials()?.into())
+    }
+
+    /// Returns an [AccessTokenCredentials] instance with the configured settings.
+    ///
+    /// # Example
+    ///
+    /// ```no_run
+    /// # use google_cloud_auth::credentials::{Builder, AccessTokenCredentials, AccessTokenCredentialsProvider};
+    /// # tokio_test::block_on(async {
+    /// // This will search for Application Default Credentials and build AccessTokenCredentials.
+    /// let credentials: AccessTokenCredentials = Builder::default()
+    ///     .build_access_token_credentials()?;
+    /// let access_token = credentials.access_token().await?;
+    /// println!("Token: {}", access_token.token);
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    ///
+    /// # Errors
+    ///
+    /// Returns a [CredentialsError] if an unsupported credential type is provided
+    /// or if the JSON value is either malformed or missing required fields.
+    ///
+    /// For more information, on how to generate the JSON for a credential,
+    /// consult the relevant section in the [application-default credentials] guide.
+    ///
+    /// [application-default credentials]: https://cloud.google.com/docs/authentication/application-default-credentials
+    pub fn build_access_token_credentials(self) -> BuildResult<AccessTokenCredentials> {
         let json_data = match load_adc()? {
             AdcContents::Contents(contents) => {
                 Some(serde_json::from_str(&contents).map_err(BuilderError::parsing)?)
@@ -415,6 +559,37 @@ impl Builder {
             .or(self.quota_project_id);
         build_credentials(json_data, quota_project_id, self.scopes)
     }
+
+    /// Returns a [crate::signer::Signer] instance with the configured settings.
+    ///
+    /// This method automatically loads Application Default Credentials (ADC)
+    /// from the environment and uses them to create a signer.
+    ///
+    /// The returned [crate::signer::Signer] might perform signing locally (e.g. if a service
+    /// account key is found) or via a remote API (e.g. if running on GCE).
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::Builder;
+    /// # use google_cloud_auth::signer::Signer;
+    /// # tokio_test::block_on(async {
+    /// let signer: Signer = Builder::default().build_signer()?;
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    pub fn build_signer(self) -> BuildResult<crate::signer::Signer> {
+        let json_data = match load_adc()? {
+            AdcContents::Contents(contents) => {
+                Some(serde_json::from_str(&contents).map_err(BuilderError::parsing)?)
+            }
+            AdcContents::FallbackToMds => None,
+        };
+        let quota_project_id = std::env::var(GOOGLE_CLOUD_QUOTA_PROJECT_VAR)
+            .ok()
+            .or(self.quota_project_id);
+        build_signer(json_data, quota_project_id, self.scopes)
+    }
 }
 
 #[derive(Debug, PartialEq)]
@@ -444,6 +619,32 @@ fn extract_credential_type(json: &Value) -> BuildResult<&str> {
 /// `mds::Builder`, `service_account::Builder`, etc.) before calling `.build()`.
 /// It helps avoid repetitive code in the `build_credentials` function.
 macro_rules! config_builder {
+    ($builder_instance:expr, $quota_project_id_option:expr, $scopes_option:expr, $apply_scopes_closure:expr) => {{
+        let builder = config_common_builder!(
+            $builder_instance,
+            $quota_project_id_option,
+            $scopes_option,
+            $apply_scopes_closure
+        );
+        builder.build_access_token_credentials()
+    }};
+}
+
+/// Applies common optional configurations (quota project ID, scopes) to a
+/// specific credential builder instance and then return a signer for it.
+macro_rules! config_signer {
+    ($builder_instance:expr, $quota_project_id_option:expr, $scopes_option:expr, $apply_scopes_closure:expr) => {{
+        let builder = config_common_builder!(
+            $builder_instance,
+            $quota_project_id_option,
+            $scopes_option,
+            $apply_scopes_closure
+        );
+        builder.build_signer()
+    }};
+}
+
+macro_rules! config_common_builder {
     ($builder_instance:expr, $quota_project_id_option:expr, $scopes_option:expr, $apply_scopes_closure:expr) => {{
         let builder = $builder_instance;
         let builder = $quota_project_id_option
@@ -454,7 +655,7 @@ macro_rules! config_builder {
             .into_iter()
             .fold(builder, |b, s| $apply_scopes_closure(b, s));
 
-        builder.build()
+        builder
     }};
 }
 
@@ -462,7 +663,7 @@ fn build_credentials(
     json: Option<Value>,
     quota_project_id: Option<String>,
     scopes: Option<Vec<String>>,
-) -> BuildResult<Credentials> {
+) -> BuildResult<AccessTokenCredentials> {
     match json {
         None => config_builder!(
             mds::Builder::from_adc(),
@@ -508,6 +709,48 @@ fn build_credentials(
     }
 }
 
+fn build_signer(
+    json: Option<Value>,
+    quota_project_id: Option<String>,
+    scopes: Option<Vec<String>>,
+) -> BuildResult<crate::signer::Signer> {
+    match json {
+        None => config_signer!(
+            mds::Builder::from_adc(),
+            quota_project_id,
+            scopes,
+            |b: mds::Builder, s: Vec<String>| b.with_scopes(s)
+        ),
+        Some(json) => {
+            let cred_type = extract_credential_type(&json)?;
+            match cred_type {
+                "authorized_user" => Err(BuilderError::not_supported(
+                    "authorized_user signer is not supported",
+                )),
+                "service_account" => config_signer!(
+                    service_account::Builder::new(json),
+                    quota_project_id,
+                    scopes,
+                    |b: service_account::Builder, s: Vec<String>| b
+                        .with_access_specifier(service_account::AccessSpecifier::from_scopes(s))
+                ),
+                "impersonated_service_account" => {
+                    config_signer!(
+                        impersonated::Builder::new(json),
+                        quota_project_id,
+                        scopes,
+                        |b: impersonated::Builder, s: Vec<String>| b.with_scopes(s)
+                    )
+                }
+                "external_account" => Err(BuilderError::not_supported(
+                    "external_account signer is not supported",
+                )),
+                _ => Err(BuilderError::unknown_type(cred_type)),
+            }
+        }
+    }
+}
+
 fn path_not_found(path: String) -> BuilderError {
     BuilderError::loading(format!(
         "{path}. {}",
@@ -613,11 +856,11 @@ pub mod testing {
 pub(crate) mod tests {
     use super::*;
     use base64::Engine;
-    use gax::backoff_policy::BackoffPolicy;
-    use gax::retry_policy::RetryPolicy;
-    use gax::retry_result::RetryResult;
-    use gax::retry_state::RetryState;
-    use gax::retry_throttler::RetryThrottler;
+    use google_cloud_gax::backoff_policy::BackoffPolicy;
+    use google_cloud_gax::retry_policy::RetryPolicy;
+    use google_cloud_gax::retry_result::RetryResult;
+    use google_cloud_gax::retry_state::RetryState;
+    use google_cloud_gax::retry_throttler::RetryThrottler;
     use mockall::mock;
     use reqwest::header::AUTHORIZATION;
     use rsa::BigUint;
@@ -628,6 +871,7 @@ pub(crate) mod tests {
     use std::sync::LazyLock;
     use test_case::test_case;
     use tokio::time::Duration;
+    use tokio::time::Instant;
 
     pub(crate) fn find_source_error<'a, T: Error + 'static>(
         error: &'a (dyn Error + 'static),
@@ -649,7 +893,7 @@ pub(crate) mod tests {
             fn on_error(
                 &self,
                 state: &RetryState,
-                error: gax::error::Error,
+                error: google_cloud_gax::error::Error,
             ) -> RetryResult;
         }
     }
@@ -763,6 +1007,16 @@ pub(crate) mod tests {
             .expect("Failed to create RsaPrivateKey from primes")
     });
 
+    #[cfg(feature = "idtoken")]
+    pub static ES256_PRIVATE_KEY: LazyLock<p256::SecretKey> = LazyLock::new(|| {
+        let secret_key_bytes = [
+            0x4c, 0x0c, 0x11, 0x6e, 0x6e, 0xb0, 0x07, 0xbd, 0x48, 0x0c, 0xc0, 0x48, 0xc0, 0x1f,
+            0xac, 0x3d, 0x82, 0x82, 0x0e, 0x6c, 0x3d, 0x76, 0x61, 0x4d, 0x06, 0x4e, 0xdb, 0x05,
+            0x26, 0x6c, 0x75, 0xdf,
+        ];
+        p256::SecretKey::from_bytes((&secret_key_bytes).into()).unwrap()
+    });
+
     pub static PKCS8_PK: LazyLock<String> = LazyLock::new(|| {
         RSA_PRIVATE_KEY
             .to_pkcs8_pem(LineEnding::LF)
@@ -994,4 +1248,19 @@ pub(crate) mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn debug_access_token() {
+        let expires_at = Instant::now() + Duration::from_secs(3600);
+        let token = Token {
+            token: "token-test-only".into(),
+            token_type: "Bearer".into(),
+            expires_at: Some(expires_at),
+            metadata: None,
+        };
+        let access_token: AccessToken = token.into();
+        let got = format!("{access_token:?}");
+        assert!(!got.contains("token-test-only"), "{got}");
+        assert!(got.contains("token: \"[censored]\""), "{got}");
+    }
 }
