@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use gax::error::CredentialsError;
+use google_cloud_gax::error::CredentialsError;
 use reqwest::Client;
 use serde::{Deserialize, Serialize};
 use serde_json::Value;
@@ -206,7 +206,7 @@ mod tests {
             .expect_err("parsing should fail");
 
         assert!(!err.is_transient(), "{err:?}");
-        assert!(err.source().is_none());
+        assert!(err.source().is_none(), "{:?}", err.source());
 
         assert!(err.to_string().contains("`access_token`"), "{err:?}");
         assert!(
