@@ -15,5 +15,5 @@
 #[allow(dead_code)]
 pub mod sts_exchange;
 
-#[cfg(google_cloud_unstable_id_token)]
+#[cfg(feature = "idtoken")]
 pub mod jwk_client;
