@@ -54,8 +54,8 @@
 //! # use http::Extensions;
 //! # use std::time::Duration;
 //! # tokio_test::block_on(async {
-//! use gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
-//! use gax::exponential_backoff::ExponentialBackoff;
+//! use google_cloud_gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
+//! use google_cloud_gax::exponential_backoff::ExponentialBackoff;
 //! let backoff = ExponentialBackoff::default();
 //! let credentials: Credentials = Builder::default()
 //!     .with_retry_policy(AlwaysRetry.with_attempt_limit(3))
@@ -74,31 +74,23 @@
 //! [gke-link]: https://cloud.google.com/kubernetes-engine
 //! [Metadata Service]: https://cloud.google.com/compute/docs/metadata/overview
 
-use crate::credentials::dynamic::CredentialsProvider;
-use crate::credentials::{CacheableResource, Credentials};
-use crate::errors::CredentialsError;
-use crate::headers_util::build_cacheable_headers;
+use crate::credentials::dynamic::{AccessTokenCredentialsProvider, CredentialsProvider};
+use crate::credentials::{AccessToken, AccessTokenCredentials, CacheableResource, Credentials};
+use crate::headers_util::AuthHeadersBuilder;
+use crate::mds::client::Client as MDSClient;
 use crate::retry::{Builder as RetryTokenProviderBuilder, TokenProviderWithRetry};
 use crate::token::{CachedTokenProvider, Token, TokenProvider};
 use crate::token_cache::TokenCache;
 use crate::{BuildResult, Result};
 use async_trait::async_trait;
-use bon::Builder;
-use gax::backoff_policy::BackoffPolicyArg;
-use gax::retry_policy::RetryPolicyArg;
-use gax::retry_throttler::RetryThrottlerArg;
-use http::{Extensions, HeaderMap, HeaderValue};
-use reqwest::Client;
+use google_cloud_gax::backoff_policy::BackoffPolicyArg;
+use google_cloud_gax::error::CredentialsError;
+use google_cloud_gax::retry_policy::RetryPolicyArg;
+use google_cloud_gax::retry_throttler::RetryThrottlerArg;
+use http::{Extensions, HeaderMap};
 use std::default::Default;
 use std::sync::Arc;
-use std::time::Duration;
-use tokio::time::Instant;
 
-pub(crate) const METADATA_FLAVOR_VALUE: &str = "Google";
-pub(crate) const METADATA_FLAVOR: &str = "metadata-flavor";
-pub(crate) const METADATA_ROOT: &str = "http://metadata.google.internal";
-pub(crate) const MDS_DEFAULT_URI: &str = "/computeMetadata/v1/instance/service-accounts/default";
-pub(crate) const GCE_METADATA_HOST_ENV_VAR: &str = "GCE_METADATA_HOST";
 // TODO(#2235) - Improve this message by talking about retries when really running with MDS
 const MDS_NOT_FOUND_ERROR: &str = concat!(
     "Could not fetch an auth token to authenticate with Google Cloud. ",
@@ -140,7 +132,7 @@ pub struct Builder {
 impl Builder {
     /// Sets the endpoint for this credentials.
     ///
-    /// A trailing slash is significant, so specify the base URL without a trailing  
+    /// A trailing slash is significant, so specify the base URL without a trailing
     /// slash. If not set, the credentials use `http://metadata.google.internal`.
     ///
     /// # Example
@@ -195,7 +187,7 @@ impl Builder {
     /// ```
     /// # use google_cloud_auth::credentials::mds::Builder;
     /// # tokio_test::block_on(async {
-    /// use gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
+    /// use google_cloud_gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
     /// let credentials = Builder::default()
     ///     .with_retry_policy(AlwaysRetry.with_attempt_limit(3))
     ///     .build();
@@ -214,7 +206,7 @@ impl Builder {
     /// # use google_cloud_auth::credentials::mds::Builder;
     /// # use std::time::Duration;
     /// # tokio_test::block_on(async {
-    /// use gax::exponential_backoff::ExponentialBackoff;
+    /// use google_cloud_gax::exponential_backoff::ExponentialBackoff;
     /// let policy = ExponentialBackoff::default();
     /// let credentials = Builder::default()
     ///     .with_backoff_policy(policy)
@@ -240,7 +232,7 @@ impl Builder {
     /// ```
     /// # use google_cloud_auth::credentials::mds::Builder;
     /// # tokio_test::block_on(async {
-    /// use gax::retry_throttler::AdaptiveThrottler;
+    /// use google_cloud_gax::retry_throttler::AdaptiveThrottler;
     /// let credentials = Builder::default()
     ///     .with_retry_throttler(AdaptiveThrottler::default())
     ///     .build();
@@ -260,28 +252,9 @@ impl Builder {
     }
 
     fn build_token_provider(self) -> TokenProviderWithRetry<MDSAccessTokenProvider> {
-        let final_endpoint: String;
-        let endpoint_overridden: bool;
-
-        // Determine the endpoint and whether it was overridden
-        if let Ok(host_from_env) = std::env::var(GCE_METADATA_HOST_ENV_VAR) {
-            // Check GCE_METADATA_HOST environment variable first
-            final_endpoint = format!("http://{host_from_env}");
-            endpoint_overridden = true;
-        } else if let Some(builder_endpoint) = self.endpoint {
-            // Else, check if an endpoint was provided to the mds::Builder
-            final_endpoint = builder_endpoint;
-            endpoint_overridden = true;
-        } else {
-            // Else, use the default metadata root
-            final_endpoint = METADATA_ROOT.to_string();
-            endpoint_overridden = false;
-        };
-
         let tp = MDSAccessTokenProvider::builder()
-            .endpoint(final_endpoint)
+            .endpoint(self.endpoint)
             .maybe_scopes(self.scopes)
-            .endpoint_overridden(endpoint_overridden)
             .created_by_adc(self.created_by_adc)
             .build();
         self.retry_builder.build(tp)
@@ -289,14 +262,73 @@ impl Builder {
 
     /// Returns a [Credentials] instance with the configured settings.
     pub fn build(self) -> BuildResult<Credentials> {
+        Ok(self.build_access_token_credentials()?.into())
+    }
+
+    /// Returns an [AccessTokenCredentials] instance with the configured settings.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::mds::Builder;
+    /// # use google_cloud_auth::credentials::{AccessTokenCredentials, AccessTokenCredentialsProvider};
+    /// # tokio_test::block_on(async {
+    /// let credentials: AccessTokenCredentials = Builder::default()
+    ///     .with_quota_project_id("my-quota-project")
+    ///     .build_access_token_credentials()?;
+    /// let access_token = credentials.access_token().await?;
+    /// println!("Token: {}", access_token.token);
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    pub fn build_access_token_credentials(self) -> BuildResult<AccessTokenCredentials> {
         let mdsc = MDSCredentials {
             quota_project_id: self.quota_project_id.clone(),
             token_provider: TokenCache::new(self.build_token_provider()),
         };
-        Ok(Credentials {
+        Ok(AccessTokenCredentials {
             inner: Arc::new(mdsc),
         })
     }
+
+    /// Returns a [crate::signer::Signer] instance with the configured settings.
+    ///
+    /// The returned [crate::signer::Signer] uses the [IAM signBlob API] to sign content. This API
+    /// requires a network request for each signing operation.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::mds::Builder;
+    /// # use google_cloud_auth::signer::Signer;
+    /// # tokio_test::block_on(async {
+    /// let signer: Signer = Builder::default().build_signer()?;
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    ///
+    /// [IAM signBlob API]: https://cloud.google.com/iam/docs/reference/credentials/rest/v1/projects.serviceAccounts/signBlob
+    pub fn build_signer(self) -> BuildResult<crate::signer::Signer> {
+        self.build_signer_with_iam_endpoint_override(None)
+    }
+
+    // only used for testing
+    fn build_signer_with_iam_endpoint_override(
+        self,
+        iam_endpoint: Option<String>,
+    ) -> BuildResult<crate::signer::Signer> {
+        let client = MDSClient::new(self.endpoint.clone());
+        let credentials = self.build()?;
+        let signing_provider = crate::signer::mds::MDSSigner::new(client, credentials);
+        let signing_provider = iam_endpoint
+            .iter()
+            .fold(signing_provider, |signing_provider, endpoint| {
+                signing_provider.with_iam_endpoint_override(endpoint)
+            });
+        Ok(crate::signer::Signer {
+            inner: Arc::new(signing_provider),
+        })
+    }
 }
 
 #[async_trait::async_trait]
@@ -305,30 +337,72 @@ where
     T: CachedTokenProvider,
 {
     async fn headers(&self, extensions: Extensions) -> Result<CacheableResource<HeaderMap>> {
-        let cached_token = self.token_provider.token(extensions).await?;
-        build_cacheable_headers(&cached_token, &self.quota_project_id)
+        let token = self.token_provider.token(extensions).await?;
+
+        AuthHeadersBuilder::new(&token)
+            .maybe_quota_project_id(self.quota_project_id.as_deref())
+            .build()
     }
 }
 
-#[derive(Clone, Debug, PartialEq, serde::Deserialize, serde::Serialize)]
-struct MDSTokenResponse {
-    access_token: String,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    expires_in: Option<u64>,
-    token_type: String,
+#[async_trait::async_trait]
+impl<T> AccessTokenCredentialsProvider for MDSCredentials<T>
+where
+    T: CachedTokenProvider,
+{
+    async fn access_token(&self) -> Result<AccessToken> {
+        let token = self.token_provider.token(Extensions::new()).await?;
+        token.into()
+    }
 }
 
-#[derive(Debug, Clone, Default, Builder)]
-struct MDSAccessTokenProvider {
-    #[builder(into)]
+#[derive(Debug, Default)]
+struct MDSAccessTokenProviderBuilder {
     scopes: Option<Vec<String>>,
-    #[builder(into)]
-    endpoint: String,
-    endpoint_overridden: bool,
+    endpoint: Option<String>,
+    created_by_adc: bool,
+}
+
+impl MDSAccessTokenProviderBuilder {
+    fn build(self) -> MDSAccessTokenProvider {
+        MDSAccessTokenProvider {
+            client: MDSClient::new(self.endpoint),
+            scopes: self.scopes,
+            created_by_adc: self.created_by_adc,
+        }
+    }
+
+    fn maybe_scopes(mut self, v: Option<Vec<String>>) -> Self {
+        self.scopes = v;
+        self
+    }
+
+    fn endpoint<T>(mut self, v: Option<T>) -> Self
+    where
+        T: Into<String>,
+    {
+        self.endpoint = v.map(Into::into);
+        self
+    }
+
+    fn created_by_adc(mut self, v: bool) -> Self {
+        self.created_by_adc = v;
+        self
+    }
+}
+
+#[derive(Debug, Clone)]
+struct MDSAccessTokenProvider {
+    scopes: Option<Vec<String>>,
+    client: MDSClient,
     created_by_adc: bool,
 }
 
 impl MDSAccessTokenProvider {
+    fn builder() -> MDSAccessTokenProviderBuilder {
+        MDSAccessTokenProviderBuilder::default()
+    }
+
     // During ADC, if no credentials are found in the well-known location and the GOOGLE_APPLICATION_CREDENTIALS
     // environment variable is not set, we default to MDS credentials without checking if the code is really
     // running in an environment with MDS. To help users who got to this state because of lack of credentials
@@ -345,55 +419,17 @@ impl MDSAccessTokenProvider {
     }
 
     fn use_adc_message(&self) -> bool {
-        self.created_by_adc && !self.endpoint_overridden
+        self.created_by_adc && self.client.is_default_endpoint
     }
 }
 
 #[async_trait]
 impl TokenProvider for MDSAccessTokenProvider {
     async fn token(&self) -> Result<Token> {
-        let client = Client::new();
-        let request = client
-            .get(format!("{}{}/token", self.endpoint, MDS_DEFAULT_URI))
-            .header(
-                METADATA_FLAVOR,
-                HeaderValue::from_static(METADATA_FLAVOR_VALUE),
-            );
-        // Use the `scopes` option if set, otherwise let the MDS use the default
-        // scopes.
-        let scopes = self.scopes.as_ref().map(|v| v.join(","));
-        let request = scopes
-            .into_iter()
-            .fold(request, |r, s| r.query(&[("scopes", s)]));
-
-        // If the connection to MDS was not successful, it is useful to retry when really
-        // running on MDS environments and not useful if there is no MDS. We will mark the error
-        // as retryable and let the retry policy determine whether to retry or not. Whenever we
-        // define a default retry policy, we can skip retrying this case.
-        let response = request
-            .send()
+        self.client
+            .access_token(self.scopes.clone())
             .await
-            .map_err(|e| crate::errors::from_http_error(e, self.error_message()))?;
-        // Process the response
-        if !response.status().is_success() {
-            let err = crate::errors::from_http_response(response, self.error_message()).await;
-            return Err(err);
-        }
-        let response = response.json::<MDSTokenResponse>().await.map_err(|e| {
-            // Decoding errors are not transient. Typically they indicate a badly
-            // configured MDS endpoint, or DNS redirecting the request to a random
-            // server, e.g., ISPs that redirect unknown services to HTTP.
-            CredentialsError::from_source(!e.is_decode(), e)
-        })?;
-        let token = Token {
-            token: response.access_token,
-            token_type: response.token_type,
-            expires_at: response
-                .expires_in
-                .map(|d| Instant::now() + Duration::from_secs(d)),
-            metadata: None,
-        };
-        Ok(token)
+            .map_err(|e| CredentialsError::new(e.is_transient(), self.error_message(), e))
     }
 }
 
@@ -409,6 +445,8 @@ mod tests {
     };
     use crate::errors;
     use crate::errors::CredentialsError;
+    use crate::mds::client::MDSTokenResponse;
+    use crate::mds::{GCE_METADATA_HOST_ENV_VAR, MDS_DEFAULT_URI, METADATA_ROOT};
     use crate::token::tests::MockTokenProvider;
     use http::HeaderValue;
     use http::header::AUTHORIZATION;
@@ -420,7 +458,9 @@ mod tests {
     use scoped_env::ScopedEnv;
     use serial_test::{parallel, serial};
     use std::error::Error;
+    use std::time::Duration;
     use test_case::test_case;
+    use tokio::time::Instant;
     use url::Url;
 
     type TestResult = anyhow::Result<()>;
@@ -443,7 +483,7 @@ mod tests {
             .build_token_provider();
 
         let err = provider.token().await.unwrap_err();
-        assert!(!err.is_transient());
+        assert!(err.is_transient(), "{err:?}");
         server.verify_and_clear();
         Ok(())
     }
@@ -508,15 +548,20 @@ mod tests {
     }
 
     #[test]
+    #[parallel]
     fn validate_default_endpoint_urls() {
         let default_endpoint_address = Url::parse(&format!("{METADATA_ROOT}{MDS_DEFAULT_URI}"));
-        assert!(default_endpoint_address.is_ok());
+        assert!(
+            default_endpoint_address.is_ok(),
+            "{default_endpoint_address:?}"
+        );
 
         let token_endpoint_address = Url::parse(&format!("{METADATA_ROOT}{MDS_DEFAULT_URI}/token"));
-        assert!(token_endpoint_address.is_ok());
+        assert!(token_endpoint_address.is_ok(), "{token_endpoint_address:?}");
     }
 
     #[tokio::test]
+    #[parallel]
     async fn headers_success() -> TestResult {
         let token = Token {
             token: "test-token".to_string(),
@@ -556,6 +601,32 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
+    async fn access_token_success() -> TestResult {
+        let server = Server::run();
+        let response = MDSTokenResponse {
+            access_token: "test-access-token".to_string(),
+            expires_in: Some(3600),
+            token_type: "Bearer".to_string(),
+        };
+        server.expect(
+            Expectation::matching(all_of![request::path(format!("{MDS_DEFAULT_URI}/token")),])
+                .respond_with(json_encoded(response)),
+        );
+
+        let creds = Builder::default()
+            .with_endpoint(format!("http://{}", server.addr()))
+            .build_access_token_credentials()
+            .unwrap();
+
+        let access_token = creds.access_token().await.unwrap();
+        assert_eq!(access_token.token, "test-access-token");
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
     async fn headers_failure() {
         let mut mock = MockTokenProvider::new();
         mock.expect_token()
@@ -566,15 +637,15 @@ mod tests {
             quota_project_id: None,
             token_provider: TokenCache::new(mock),
         };
-        assert!(mdsc.headers(Extensions::new()).await.is_err());
+        let result = mdsc.headers(Extensions::new()).await;
+        assert!(result.is_err(), "{result:?}");
     }
 
     #[test]
+    #[parallel]
     fn error_message_with_adc() {
         let provider = MDSAccessTokenProvider::builder()
-            .endpoint("http://127.0.0.1")
             .created_by_adc(true)
-            .endpoint_overridden(false)
             .build();
 
         let want = MDS_NOT_FOUND_ERROR;
@@ -586,10 +657,14 @@ mod tests {
     #[test_case(false, true)]
     #[test_case(true, true)]
     fn error_message_without_adc(adc: bool, overridden: bool) {
+        let endpoint = if overridden {
+            Some("http://127.0.0.1")
+        } else {
+            None
+        };
         let provider = MDSAccessTokenProvider::builder()
-            .endpoint("http://127.0.0.1")
+            .endpoint(endpoint)
             .created_by_adc(adc)
-            .endpoint_overridden(overridden)
             .build();
 
         let not_want = MDS_NOT_FOUND_ERROR;
@@ -617,7 +692,7 @@ mod tests {
     #[tokio::test]
     #[serial]
     async fn adc_overridden_mds() -> TestResult {
-        let _e = ScopedEnv::set(super::GCE_METADATA_HOST_ENV_VAR, "metadata.overridden");
+        let _e = ScopedEnv::set(GCE_METADATA_HOST_ENV_VAR, "metadata.overridden");
 
         let err = Builder::from_adc()
             .build_token_provider()
@@ -625,7 +700,7 @@ mod tests {
             .await
             .unwrap_err();
 
-        let _e = ScopedEnv::remove(super::GCE_METADATA_HOST_ENV_VAR);
+        let _e = ScopedEnv::remove(GCE_METADATA_HOST_ENV_VAR);
 
         let original_err = find_source_error::<CredentialsError>(&err).unwrap();
         assert!(original_err.is_transient());
@@ -675,13 +750,13 @@ mod tests {
         );
 
         let addr = server.addr().to_string();
-        let _e = ScopedEnv::set(super::GCE_METADATA_HOST_ENV_VAR, &addr);
+        let _e = ScopedEnv::set(GCE_METADATA_HOST_ENV_VAR, &addr);
         let mdsc = Builder::default()
             .with_scopes(["scope1", "scope2"])
             .build()
             .unwrap();
         let headers = mdsc.headers(Extensions::new()).await.unwrap();
-        let _e = ScopedEnv::remove(super::GCE_METADATA_HOST_ENV_VAR);
+        let _e = ScopedEnv::remove(GCE_METADATA_HOST_ENV_VAR);
 
         assert_eq!(
             get_token_from_headers(headers).unwrap(),
@@ -958,9 +1033,57 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
     async fn get_default_universe_domain_success() -> TestResult {
         let universe_domain_response = Builder::default().build()?.universe_domain().await.unwrap();
         assert_eq!(universe_domain_response, DEFAULT_UNIVERSE_DOMAIN);
         Ok(())
     }
+
+    #[tokio::test]
+    #[parallel]
+    async fn get_mds_signer() -> TestResult {
+        use base64::{Engine, prelude::BASE64_STANDARD};
+        use serde_json::json;
+
+        let server = Server::run();
+        server.expect(
+            Expectation::matching(all_of![request::path(format!("{MDS_DEFAULT_URI}/token")),])
+                .respond_with(json_encoded(MDSTokenResponse {
+                    access_token: "test-access-token".to_string(),
+                    expires_in: None,
+                    token_type: "Bearer".to_string(),
+                })),
+        );
+        server.expect(
+            Expectation::matching(all_of![request::path(format!("{MDS_DEFAULT_URI}/email")),])
+                .respond_with(status_code(200).body("test-client-email")),
+        );
+        server.expect(
+            Expectation::matching(all_of![
+                request::method_path(
+                    "POST",
+                    "/v1/projects/-/serviceAccounts/test-client-email:signBlob"
+                ),
+                request::headers(contains(("authorization", "Bearer test-access-token"))),
+            ])
+            .respond_with(json_encoded(json!({
+                "signedBlob": BASE64_STANDARD.encode("signed_blob"),
+            }))),
+        );
+
+        let endpoint = server.url("").to_string().trim_end_matches('/').to_string();
+
+        let signer = Builder::default()
+            .with_endpoint(&endpoint)
+            .build_signer_with_iam_endpoint_override(Some(endpoint))?;
+
+        let client_email = signer.client_email().await?;
+        assert_eq!(client_email, "test-client-email");
+
+        let signature = signer.sign(b"test").await?;
+        assert_eq!(signature.as_ref(), b"signed_blob");
+
+        Ok(())
+    }
 }
