@@ -70,14 +70,14 @@
 //! [Service Account]: https://cloud.google.com/iam/docs/service-account-overview
 //! [service account key]: https://cloud.google.com/iam/docs/keys-create-delete#creating
 
-mod jws;
+pub(crate) mod jws;
 
 use crate::build_errors::Error as BuilderError;
 use crate::constants::DEFAULT_SCOPE;
-use crate::credentials::dynamic::CredentialsProvider;
-use crate::credentials::{CacheableResource, Credentials};
-use crate::errors::{self, CredentialsError};
-use crate::headers_util::build_cacheable_headers;
+use crate::credentials::dynamic::{AccessTokenCredentialsProvider, CredentialsProvider};
+use crate::credentials::{AccessToken, AccessTokenCredentials, CacheableResource, Credentials};
+use crate::errors::{self};
+use crate::headers_util::AuthHeadersBuilder;
 use crate::token::{CachedTokenProvider, Token, TokenProvider};
 use crate::token_cache::TokenCache;
 use crate::{BuildResult, Result};
@@ -86,7 +86,7 @@ use http::{Extensions, HeaderMap};
 use jws::{CLOCK_SKEW_FUDGE, DEFAULT_TOKEN_TIMEOUT, JwsClaims, JwsHeader};
 use rustls::crypto::CryptoProvider;
 use rustls::sign::Signer;
-use rustls_pemfile::Item;
+use rustls_pki_types::{PrivateKeyDer, pem::PemObject};
 use serde_json::Value;
 use std::sync::Arc;
 use time::OffsetDateTime;
@@ -278,22 +278,102 @@ impl Builder {
     ///
     /// # Errors
     ///
-    /// Returns a [CredentialsError] if the `service_account_key`
+    /// Returns an error if the `service_account_key`
     /// provided to [`Builder::new`] cannot be successfully deserialized into the
     /// expected format for a service account key. This typically happens if the
-    /// JSON value is malformed or missing required fields. For more information,
-    /// on the expected format for a service account key, consult the
-    /// relevant section in the [service account keys] guide.
+    /// JSON value is malformed or missing required fields.
+    ///
+    /// For more information, on the expected format for a service account key,
+    /// consult the relevant section in the [service account keys] guide.
     ///
     /// [creating service account keys]: https://cloud.google.com/iam/docs/keys-create-delete#creating
     pub fn build(self) -> BuildResult<Credentials> {
-        Ok(Credentials {
+        Ok(self.build_access_token_credentials()?.into())
+    }
+
+    /// Returns an [AccessTokenCredentials] instance with the configured settings.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::service_account::Builder;
+    /// # use google_cloud_auth::credentials::{AccessTokenCredentials, AccessTokenCredentialsProvider};
+    /// # use serde_json::json;
+    /// # tokio_test::block_on(async {
+    /// let service_account_key = json!({
+    ///     "client_email": "test-client-email",
+    ///     "private_key_id": "test-private-key-id",
+    ///     "private_key": "-----BEGIN PRIVATE KEY-----\nBLAHBLAHBLAH\n-----END PRIVATE KEY-----\n",
+    ///     "project_id": "test-project-id",
+    ///     "universe_domain": "test-universe-domain",
+    /// });
+    /// let credentials: AccessTokenCredentials = Builder::new(service_account_key)
+    ///     .with_quota_project_id("my-quota-project")
+    ///     .build_access_token_credentials()?;
+    /// let access_token = credentials.access_token().await?;
+    /// println!("Token: {}", access_token.token);
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if the `service_account_key`
+    /// provided to [`Builder::new`] cannot be successfully deserialized into the
+    /// expected format for a service account key. This typically happens if the
+    /// JSON value is malformed or missing required fields.
+    ///
+    /// For more information, on the expected format for a service account key,
+    /// consult the relevant section in the [service account keys] guide.
+    ///
+    /// [service account keys]: https://cloud.google.com/iam/docs/keys-create-delete#creating
+    pub fn build_access_token_credentials(self) -> BuildResult<AccessTokenCredentials> {
+        Ok(AccessTokenCredentials {
             inner: Arc::new(ServiceAccountCredentials {
                 quota_project_id: self.quota_project_id.clone(),
                 token_provider: TokenCache::new(self.build_token_provider()?),
             }),
         })
     }
+
+    /// Returns a [crate::signer::Signer] instance with the configured settings.
+    ///
+    /// The returned [crate::signer::Signer] uses the service account's private key to sign blobs locally.
+    /// It does not make any network requests to perform signing operations.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::service_account::Builder;
+    /// # use google_cloud_auth::signer::Signer;
+    /// # use serde_json::json;
+    /// # tokio_test::block_on(async {
+    /// let service_account_key = json!({
+    ///     "client_email": "test-client-email",
+    ///     "private_key_id": "test-private-key-id",
+    ///     "private_key": "-----BEGIN PRIVATE KEY-----\nBLAHBLAHBLAH\n-----END PRIVATE KEY-----\n",
+    ///     "project_id": "test-project-id",
+    /// });
+    ///
+    /// let signer: Signer = Builder::new(service_account_key).build_signer()?;
+    /// # Ok::<(), anyhow::Error>(())
+    /// # });
+    /// ```
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if the `service_account_key` provided to [`Builder::new`]
+    /// cannot be successfully deserialized or doesn't contain a valid private key.
+    pub fn build_signer(self) -> BuildResult<crate::signer::Signer> {
+        let service_account_key =
+            serde_json::from_value::<ServiceAccountKey>(self.service_account_key.clone())
+                .map_err(BuilderError::parsing)?;
+        let signing_provider =
+            crate::signer::service_account::ServiceAccountSigner::new(service_account_key);
+        Ok(crate::signer::Signer {
+            inner: Arc::new(signing_provider),
+        })
+    }
 }
 
 /// A representation of a [service account key].
@@ -303,7 +383,7 @@ impl Builder {
 pub(crate) struct ServiceAccountKey {
     /// The client email address of the service account.
     /// (e.g., "my-sa@my-project.iam.gserviceaccount.com").
-    client_email: String,
+    pub(crate) client_email: String,
     /// ID of the service account's private key.
     private_key_id: String,
     /// The PEM-encoded PKCS#8 private key string associated with the service account.
@@ -315,6 +395,54 @@ pub(crate) struct ServiceAccountKey {
     universe_domain: Option<String>,
 }
 
+impl ServiceAccountKey {
+    // Creates a signer using the private key stored in the service account file.
+    pub(crate) fn signer(&self) -> Result<Box<dyn Signer>> {
+        let private_key = self.private_key.clone();
+        let key_provider = CryptoProvider::get_default().map(|p| p.key_provider);
+        #[cfg(feature = "default-rustls-provider")]
+        let key_provider = key_provider
+            .unwrap_or_else(|| rustls::crypto::aws_lc_rs::default_provider().key_provider);
+        #[cfg(not(feature = "default-rustls-provider"))]
+        let key_provider = key_provider.expect(
+            r###"
+The default rustls::CryptoProvider should be configured by the application. The
+`google-cloud-auth` crate was compiled without the `default-rustls-provider`
+feature. Without this feature the crate expects the application to initialize
+the rustls crypto provider using `rustls::CryptoProvider::install_default()`.
+
+Note that the application must use the exact same version of `rustls` as the
+`google-cloud-auth` crate does. Otherwise `install_default()` has no effect."###,
+        );
+
+        let key_der = PrivateKeyDer::from_pem_slice(private_key.as_bytes()).map_err(|e| {
+            errors::non_retryable_from_str(format!(
+                "Failed to parse service account private key PEM: {}",
+                e
+            ))
+        })?;
+
+        let pkcs8_der = match key_der {
+            PrivateKeyDer::Pkcs8(der) => der,
+            _ => {
+                return Err(errors::non_retryable_from_str(format!(
+                    "expected key to be in form of PKCS8, found {:?}",
+                    key_der
+                )));
+            }
+        };
+
+        let pk = key_provider
+            .load_private_key(PrivateKeyDer::Pkcs8(pkcs8_der))
+            .map_err(errors::non_retryable)?;
+
+        pk.choose_scheme(&[rustls::SignatureScheme::RSA_PKCS1_SHA256])
+            .ok_or_else(||{
+                errors::non_retryable_from_str("Unable to choose RSA_PKCS1_SHA256 signing scheme as it is not supported by current signer")
+            })
+    }
+}
+
 impl std::fmt::Debug for ServiceAccountKey {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("ServiceAccountKey")
@@ -385,7 +513,7 @@ pub(crate) struct ServiceAccountTokenGenerator {
 }
 
 impl ServiceAccountTokenGenerator {
-    #[cfg(google_cloud_unstable_id_token)]
+    #[cfg(feature = "idtoken")]
     pub(crate) fn new_id_token_generator(
         target_audience: String,
         audience: String,
@@ -400,7 +528,7 @@ impl ServiceAccountTokenGenerator {
     }
 
     pub(crate) fn generate(&self) -> Result<String> {
-        let signer = self.signer(&self.service_account_key.private_key)?;
+        let signer = self.service_account_key.signer()?;
 
         // The claims encode a unix timestamp. `std::time::Instant` has no
         // epoch, so we use `time::OffsetDateTime`, which reads system time, in
@@ -421,7 +549,7 @@ impl ServiceAccountTokenGenerator {
         let header = JwsHeader {
             alg: "RS256",
             typ: "JWT",
-            kid: &self.service_account_key.private_key_id,
+            kid: Some(self.service_account_key.private_key_id.clone()),
         };
         let encoded_header_claims = format!("{}.{}", header.encode()?, claims.encode()?);
         let sig = signer
@@ -436,35 +564,6 @@ impl ServiceAccountTokenGenerator {
 
         Ok(token)
     }
-
-    // Creates a signer using the private key stored in the service account file.
-    fn signer(&self, private_key: &String) -> Result<Box<dyn Signer>> {
-        let key_provider = CryptoProvider::get_default().map_or_else(
-            || rustls::crypto::ring::default_provider().key_provider,
-            |p| p.key_provider,
-        );
-
-        let private_key = rustls_pemfile::read_one(&mut private_key.as_bytes())
-            .map_err(errors::non_retryable)?
-            .ok_or_else(|| {
-                errors::non_retryable_from_str("missing PEM section in service account key")
-            })?;
-        let pk = match private_key {
-            Item::Pkcs8Key(item) => key_provider.load_private_key(item.into()),
-            other => {
-                return Err(Self::unexpected_private_key_error(other));
-            }
-        };
-        let sk = pk.map_err(errors::non_retryable)?;
-        sk.choose_scheme(&[rustls::SignatureScheme::RSA_PKCS1_SHA256])
-            .ok_or_else(|| errors::non_retryable_from_str("Unable to choose RSA_PKCS1_SHA256 signing scheme as it is not supported by current signer"))
-    }
-
-    fn unexpected_private_key_error(private_key_format: Item) -> CredentialsError {
-        errors::non_retryable_from_str(format!(
-            "expected key to be in form of PKCS8, found {private_key_format:?}",
-        ))
-    }
 }
 
 #[async_trait::async_trait]
@@ -474,7 +573,21 @@ where
 {
     async fn headers(&self, extensions: Extensions) -> Result<CacheableResource<HeaderMap>> {
         let token = self.token_provider.token(extensions).await?;
-        build_cacheable_headers(&token, &self.quota_project_id)
+
+        AuthHeadersBuilder::new(&token)
+            .maybe_quota_project_id(self.quota_project_id.as_deref())
+            .build()
+    }
+}
+
+#[async_trait::async_trait]
+impl<T> AccessTokenCredentialsProvider for ServiceAccountCredentials<T>
+where
+    T: CachedTokenProvider,
+{
+    async fn access_token(&self) -> Result<AccessToken> {
+        let token = self.token_provider.token(Extensions::new()).await?;
+        token.into()
     }
 }
 
@@ -490,7 +603,6 @@ mod tests {
     use http::header::AUTHORIZATION;
     use rsa::pkcs1::EncodeRsaPrivateKey;
     use rsa::pkcs8::LineEnding;
-    use rustls_pemfile::Item;
     use serde_json::Value;
     use serde_json::json;
     use std::error::Error as _;
@@ -616,7 +728,8 @@ mod tests {
             token_provider: TokenCache::new(mock),
             quota_project_id: None,
         };
-        assert!(sac.headers(Extensions::new()).await.is_err());
+        let result = sac.headers(Extensions::new()).await;
+        assert!(result.is_err(), "{result:?}");
     }
 
     fn get_mock_service_key() -> Value {
@@ -639,7 +752,7 @@ mod tests {
 
         service_account_key["private_key"] = Value::from(key);
         let cred = Builder::new(service_account_key).build()?;
-        let expected_error_message = "expected key to be in form of PKCS8, found Pkcs1Key";
+        let expected_error_message = "expected key to be in form of PKCS8, found ";
         assert!(
             cred.headers(Extensions::new())
                 .await
@@ -753,23 +866,29 @@ mod tests {
             ..Default::default()
         };
 
-        let signer = tg.signer(&tg.service_account_key.private_key);
-        let expected_error_message = "missing PEM section in service account key";
+        let signer = tg.service_account_key.signer();
+        let expected_error_message = "Failed to parse service account private key PEM";
         assert!(signer.is_err_and(|e| e.to_string().contains(expected_error_message)));
         Ok(())
     }
 
     #[test]
-    fn unexpected_private_key_error_message() -> TestResult {
-        let expected_message = format!(
-            "expected key to be in form of PKCS8, found {:?}",
-            Item::Crl(Vec::new().into()) // Example unsupported key type
+    fn signer_fails_on_invalid_pem_type() -> TestResult {
+        let invalid_pem = concat!(
+            "-----BEGI X509 CRL-----\n",
+            "MIIBmzCBja... (truncated) ...\n",
+            "-----END X509 CRL-----"
         );
 
-        let error = ServiceAccountTokenGenerator::unexpected_private_key_error(Item::Crl(
-            Vec::new().into(),
-        ));
-        assert!(error.to_string().contains(&expected_message));
+        let mut key = ServiceAccountKey {
+            private_key: invalid_pem.to_string(),
+            ..Default::default()
+        };
+        key.private_key = invalid_pem.to_string();
+        let result = key.signer();
+        assert!(result.is_err(), "{result:?}");
+        let error_msg = result.unwrap_err().to_string();
+        assert!(error_msg.contains("Failed to parse service account private key PEM"));
         Ok(())
     }
 
@@ -851,4 +970,39 @@ mod tests {
         assert_eq!(claims["sub"], service_account_key["client_email"]);
         Ok(())
     }
+
+    #[tokio::test]
+    async fn get_service_account_access_token() -> TestResult {
+        let mut service_account_key = get_mock_service_key();
+        service_account_key["private_key"] = Value::from(PKCS8_PK.clone());
+        let creds = Builder::new(service_account_key.clone()).build_access_token_credentials()?;
+
+        let access_token = creds.access_token().await?;
+        let token = access_token.token;
+
+        let re = regex::Regex::new(SSJ_REGEX).unwrap();
+        let captures = re.captures(&token).ok_or_else(|| {
+            format!(r#"Expected token in form: "<header>.<claims>.<sig>". Found token: {token}"#)
+        })?;
+        let token_header = b64_decode_to_json(captures["header"].to_string());
+        assert_eq!(token_header["alg"], "RS256");
+        assert_eq!(token_header["typ"], "JWT");
+        assert_eq!(token_header["kid"], service_account_key["private_key_id"]);
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn get_service_account_signer() -> TestResult {
+        let mut service_account_key = get_mock_service_key();
+        service_account_key["private_key"] = Value::from(PKCS8_PK.clone());
+        let signer = Builder::new(service_account_key.clone()).build_signer()?;
+
+        let client_email = signer.client_email().await?;
+        assert_eq!(client_email, service_account_key["client_email"]);
+
+        let _bytes = signer.sign(b"test").await?;
+
+        Ok(())
+    }
 }
