@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+//! The token cache
+
 use crate::Result;
 use crate::credentials::{CacheableResource, EntityTag};
 use crate::token::{CachedTokenProvider, Token, TokenProvider};
@@ -27,36 +29,22 @@ use tokio::time::{Duration, Instant, sleep};
 const NORMAL_REFRESH_SLACK: Duration = Duration::from_secs(240);
 const SHORT_REFRESH_SLACK: Duration = Duration::from_secs(10);
 
-#[derive(Debug)]
-pub(crate) struct TokenCache<T: TokenProvider> {
+#[derive(Debug, Clone)]
+pub(crate) struct TokenCache {
     rx_token: watch::Receiver<Option<Result<(Token, EntityTag)>>>,
-    token_provider: Arc<T>,
 }
 
-// The default implementation requires `T` to implement `Clone`, which is not always the case.
-impl<T: TokenProvider> Clone for TokenCache<T> {
-    fn clone(&self) -> Self {
-        Self {
-            rx_token: self.rx_token.clone(),
-            token_provider: self.token_provider.clone(),
-        }
-    }
-}
-
-impl<T> TokenCache<T>
-where
-    T: TokenProvider + Send + Sync + 'static,
-{
-    pub(crate) fn new(inner: T) -> Self {
+impl TokenCache {
+    pub(crate) fn new<T>(inner: T) -> Self
+    where
+        T: TokenProvider + Send + Sync + 'static,
+    {
         let (tx_token, rx_token) = watch::channel::<Option<Result<(Token, EntityTag)>>>(None);
         let token_provider = Arc::new(inner);
 
-        tokio::spawn(refresh_task(token_provider.clone(), tx_token));
+        tokio::spawn(refresh_task(token_provider, tx_token));
 
-        Self {
-            rx_token,
-            token_provider,
-        }
+        Self { rx_token }
     }
 
     async fn latest_token_and_entity_tag(&self) -> Result<(Token, EntityTag)> {
@@ -86,10 +74,7 @@ where
 }
 
 #[async_trait::async_trait]
-impl<T> CachedTokenProvider for TokenCache<T>
-where
-    T: TokenProvider + Send + Sync + 'static,
-{
+impl CachedTokenProvider for TokenCache {
     async fn token(&self, extensions: Extensions) -> Result<CacheableResource<Token>> {
         let (data, entity_tag) = self.latest_token_and_entity_tag().await?;
         match extensions.get::<EntityTag>() {
@@ -116,8 +101,8 @@ async fn refresh_task<T>(
 {
     loop {
         let token_result = token_provider.token().await;
-        let expiry = token_result.as_ref().ok().map(|t| t.expires_at);
-        let tagged = token_result.map(|token| {
+        let expiry = token_result.as_ref().map(|t| t.expires_at);
+        let tagged = token_result.clone().map(|token| {
             let entity_tag = EntityTag::new();
             (token, entity_tag)
         });
@@ -125,7 +110,7 @@ async fn refresh_task<T>(
         let _ = tx_token.send(Some(tagged));
 
         match expiry {
-            Some(Some(expiry)) => {
+            Ok(Some(expiry)) => {
                 let time_until_expiry = expiry.checked_duration_since(Instant::now());
 
                 match time_until_expiry {
@@ -144,15 +129,27 @@ async fn refresh_task<T>(
                     }
                 }
             }
-            Some(None) => {
+            Ok(None) => {
                 // If there is no expiry, the token is valid forever, so no need to refresh
                 // TODO(#1553): Validate that all auth backends provide expiry and make expiry not optional.
                 break;
             }
-            None => {
-                // The retry policy has been used already by the inner token provider.
-                // If it ended in an error, just quit the background task.
-                break;
+            Err(err) => {
+                // On transient errors, even if the retry policy is exhausted,
+                // we want to continue running this retry loop.
+                // This loop cannot stop because that may leave the
+                // credentials in an unrecoverable state (see #4541).
+                // We considered using a notification to wake up the next time
+                // a caller wants to retrieve a token, but that seemed prone to
+                // deadlocks. We may implement this as an improvement (#4593).
+                // On permanent errors, then there is really no point in trying
+                // again, by definition of "permanent". If the error was misclassified
+                // as permanent, that is a bug in the retry policy and better fixed
+                // there than implemented as a workaround here.
+                if !err.is_transient() {
+                    break;
+                }
+                sleep(SHORT_REFRESH_SLACK).await;
             }
         }
     }
@@ -163,7 +160,7 @@ mod tests {
     use super::*;
     use crate::errors;
     use crate::token::tests::MockTokenProvider;
-    use gax::error::CredentialsError;
+    use google_cloud_gax::error::CredentialsError;
     use std::ops::{Add, Sub};
     use std::sync::{Arc, Mutex};
     use tokio::time::{Duration, Instant};
@@ -232,11 +229,13 @@ mod tests {
             .returning(|| Err(errors::non_retryable_from_str("fail")));
 
         let cache = TokenCache::new(mock);
-        assert!(cache.token(Extensions::new()).await.is_err());
+        let result = cache.token(Extensions::new()).await;
+        assert!(result.is_err(), "{result:?}");
 
         // Verify that a new request is made to the mock token provider when we
         // don't have a valid token.
-        assert!(cache.token(Extensions::new()).await.is_err());
+        let result = cache.token(Extensions::new()).await;
+        assert!(result.is_err(), "{result:?}");
     }
 
     #[tokio::test(start_paused = true)]
@@ -315,7 +314,8 @@ mod tests {
         tokio::time::advance(sleep).await;
 
         // make sure we return the error, not the expired token
-        assert!(cache.token(Extensions::new()).await.is_err());
+        let result = cache.token(Extensions::new()).await;
+        assert!(result.is_err(), "{result:?}");
         Ok(())
     }
 
@@ -349,9 +349,8 @@ mod tests {
 
         // Wait for the N token requests to complete, verifying the returned token.
         for task in tasks {
-            let actual = task.await.unwrap();
-            assert!(actual.is_ok(), "{}", actual.err().unwrap());
-            assert_eq!(get_cached_token(actual.unwrap())?, token);
+            let actual = task.await??;
+            assert_eq!(get_cached_token(actual)?, token);
         }
         Ok(())
     }
@@ -448,7 +447,7 @@ mod tests {
 
         // check that channel has None before refresh task starts
         let actual = rx.borrow().clone();
-        assert!(actual.is_none());
+        assert!(actual.is_none(), "{actual:?}");
 
         tokio::spawn(async move {
             refresh_task(Arc::new(mock), tx).await;
@@ -522,7 +521,7 @@ mod tests {
 
         // check that channel has None before refresh task starts
         let actual = rx.borrow().clone();
-        assert!(actual.is_none());
+        assert!(actual.is_none(), "{actual:?}");
 
         tokio::spawn(async move {
             refresh_task(Arc::new(mock), tx).await;
@@ -587,7 +586,7 @@ mod tests {
 
         // check that channel has None before refresh task starts
         let actual = rx.borrow().clone();
-        assert!(actual.is_none());
+        assert!(actual.is_none(), "{actual:?}");
 
         tokio::spawn(async move {
             refresh_task(Arc::new(mock), tx).await;
@@ -609,6 +608,64 @@ mod tests {
         assert_eq!(actual, token2);
     }
 
+    #[tokio::test(start_paused = true)]
+    async fn refresh_task_sleeps_on_transient_error_and_recovers_on_next_loop() -> TestResult {
+        let now = Instant::now();
+
+        let token = Token {
+            token: "token-1".to_string(),
+            token_type: "Bearer".to_string(),
+            expires_at: Some(now + TOKEN_VALID_DURATION),
+            metadata: None,
+        };
+
+        let mut mock = MockTokenProvider::new();
+        // 1st request succeeds
+        mock.expect_token()
+            .times(1)
+            .return_once(move || Ok(token.clone()));
+
+        // 2nd request (triggered by refresh loop) fails with transient error
+        mock.expect_token()
+            .times(1)
+            .return_once(|| Err(CredentialsError::from_msg(true, "transient error")));
+
+        let token = Token {
+            token: "token-2".to_string(),
+            token_type: "Bearer".to_string(),
+            expires_at: Some(now + 2 * TOKEN_VALID_DURATION),
+            metadata: None,
+        };
+
+        // 3rd request (triggered by next loop) succeeds
+        mock.expect_token()
+            .times(1)
+            .return_once(move || Ok(token.clone()));
+
+        let cache = TokenCache::new(mock);
+
+        // fetch an initial token
+        let actual = get_cached_token(cache.token(Extensions::new()).await.unwrap())?;
+        assert_eq!(actual.token, "token-1");
+
+        // advance time to force expiration, which wakes up the background task.
+        let sleep = TOKEN_VALID_DURATION.add(Duration::from_secs(10));
+        tokio::time::advance(sleep).await;
+
+        let result = cache.token(Extensions::new()).await;
+        assert!(result.is_err());
+        assert!(result.unwrap_err().to_string().contains("transient error"));
+
+        // Wait another SHORT_REFRESH_SLACK + buffer for the background loop to try again and recover
+        tokio::time::advance(SHORT_REFRESH_SLACK.add(Duration::from_secs(10))).await;
+        tokio::task::yield_now().await;
+
+        let actual = get_cached_token(cache.token(Extensions::new()).await.unwrap())?;
+        assert_eq!(actual.token, "token-2");
+
+        Ok(())
+    }
+
     #[derive(Clone, Debug)]
     struct FakeTokenProvider {
         result: Result<Token>,
@@ -666,9 +723,8 @@ mod tests {
 
         // Wait for the N token requests to complete, verifying the returned token.
         for task in tasks {
-            let actual = task.await?;
-            assert!(actual.is_ok(), "{}", actual.unwrap_err());
-            assert_eq!(get_cached_token(actual?)?, token);
+            let actual = task.await??;
+            assert_eq!(get_cached_token(actual)?, token);
         }
 
         let calls = tp.calls();
@@ -724,6 +780,5 @@ mod tests {
 
         assert!(debug_output.contains("TokenCache"));
         assert!(debug_output.contains("rx_token"));
-        assert!(debug_output.contains("token_provider: MockTokenProvider")); // Check for MockTokenProvider specific output part
     }
 }
