@@ -14,30 +14,30 @@
 
 #[cfg(test)]
 mod tests {
-    use std::error::Error;
-    use std::fmt;
-
+    use google_cloud_auth::credentials::EntityTag;
     use google_cloud_auth::credentials::external_account::ProgrammaticBuilder;
+    use google_cloud_auth::credentials::impersonated::Builder as ImpersonatedBuilder;
+    use google_cloud_auth::credentials::mds::Builder as MdsBuilder;
+    use google_cloud_auth::credentials::service_account::Builder as ServiceAccountBuilder;
     use google_cloud_auth::credentials::subject_token::{
         Builder as SubjectTokenBuilder, SubjectToken, SubjectTokenProvider,
     };
-    use google_cloud_auth::errors::SubjectTokenProviderError;
-
-    use google_cloud_auth::credentials::EntityTag;
-    use google_cloud_auth::credentials::mds::Builder as MdsBuilder;
-    use google_cloud_auth::credentials::service_account::Builder as ServiceAccountBuilder;
     use google_cloud_auth::credentials::user_account::Builder as UserAccountCredentialBuilder;
+    use google_cloud_auth::credentials::{AccessToken, AccessTokenCredentials};
     use google_cloud_auth::credentials::{
-        Builder as AccessTokenCredentialBuilder, CacheableResource, Credentials,
-        CredentialsProvider, api_key_credentials::Builder as ApiKeyCredentialsBuilder,
+        AccessTokenCredentialsProvider, Builder as AccessTokenCredentialBuilder, CacheableResource,
+        Credentials, CredentialsProvider, api_key_credentials::Builder as ApiKeyCredentialsBuilder,
     };
     use google_cloud_auth::errors::CredentialsError;
+    use google_cloud_auth::errors::SubjectTokenProviderError;
     use http::header::{AUTHORIZATION, HeaderName, HeaderValue};
     use http::{Extensions, HeaderMap};
     use httptest::{Expectation, Server, matchers::*, responders::*};
     use scoped_env::ScopedEnv;
     use serde_json::json;
-    use serial_test::serial;
+    use serial_test::{parallel, serial};
+    use std::error::Error;
+    use std::fmt;
     use test_case::test_case;
 
     type Result<T> = anyhow::Result<T>;
@@ -258,6 +258,7 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
     async fn create_api_key_credentials_success() {
         let creds = ApiKeyCredentialsBuilder::new("test-api-key").build();
         let fmt = format!("{creds:?}");
@@ -409,9 +410,14 @@ mod tests {
             async fn headers(&self, extensions: Extensions) -> std::result::Result<CacheableResource<HeaderMap>, CredentialsError>;
             async fn universe_domain(&self) -> Option<String>;
         }
+
+        impl AccessTokenCredentialsProvider for Credentials {
+            async fn access_token(&self) -> std::result::Result<AccessToken, CredentialsError>;
+        }
     }
 
     #[tokio::test]
+    #[parallel]
     async fn mocking_with_default_values() -> Result<()> {
         let mut mock = MockCredentials::new();
         mock.expect_headers().return_once(|_extensions| {
@@ -427,7 +433,7 @@ mod tests {
         match cached_headers {
             CacheableResource::New { entity_tag, data } => {
                 assert_eq!(entity_tag, EntityTag::default());
-                assert!(data.is_empty());
+                assert!(data.is_empty(), "{data:?}");
             }
             CacheableResource::NotModified => {
                 unreachable!("Expecting a header to be present");
@@ -439,6 +445,24 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
+    async fn mocking_access_token_credentials() -> Result<()> {
+        let mut mock = MockCredentials::new();
+        mock.expect_access_token().return_once(|| {
+            Ok(AccessToken {
+                token: "test-token".to_string(),
+            })
+        });
+
+        let creds = AccessTokenCredentials::from(mock);
+        let access_token = creds.access_token().await?;
+        assert_eq!(access_token.token, "test-token");
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
     async fn mocking_with_custom_header() -> Result<()> {
         let mut mock = MockCredentials::new();
         let headers = HeaderMap::from_iter([(
@@ -465,6 +489,7 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
     async fn mocking_with_custom_universe_domain() -> Result<()> {
         let mut mock = MockCredentials::new();
 
@@ -483,7 +508,7 @@ mod tests {
         match creds.headers(Extensions::new()).await? {
             CacheableResource::New { entity_tag, data } => {
                 assert_eq!(entity_tag, EntityTag::default());
-                assert!(data.is_empty());
+                assert!(data.is_empty(), "{data:?}");
             }
             CacheableResource::NotModified => {
                 unreachable!("Expecting a header to be present");
@@ -495,6 +520,7 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
     async fn get_mds_credentials_from_builder() -> Result<()> {
         let test_quota_project = "test-quota-project";
         let mdcs = MdsBuilder::default()
@@ -507,6 +533,7 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
     async fn get_service_account_credentials_from_builder() -> Result<()> {
         let test_quota_project = "test-quota-project";
         let service_account_info_json = json!({
@@ -526,6 +553,104 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
+    async fn get_mds_access_token_credentials_from_builder() -> Result<()> {
+        let server = Server::run();
+        let response = json!({
+            "access_token": "test-access-token",
+            "expires_in": 3600,
+            "token_type": "Bearer"
+        });
+        server.expect(
+            Expectation::matching(request::path(
+                "/computeMetadata/v1/instance/service-accounts/default/token",
+            ))
+            .respond_with(json_encoded(response)),
+        );
+
+        let test_quota_project = "test-quota-project";
+        let mdcs = MdsBuilder::default()
+            .with_endpoint(server.url("").to_string().trim_end_matches("/"))
+            .with_quota_project_id(test_quota_project)
+            .build_access_token_credentials()?;
+
+        let fmt = format!("{mdcs:?}");
+        assert!(fmt.contains("MDSCredentials"));
+        assert!(fmt.contains(test_quota_project));
+
+        let access_token = mdcs.access_token().await?;
+        assert_eq!(access_token.token, "test-access-token");
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
+    async fn get_service_account_access_token_credentials_from_builder() -> Result<()> {
+        let test_quota_project = "test-quota-project";
+        let service_account_info_json = json!({
+            "client_email": "test-client-email",
+            "private_key_id": "test-private-key-id",
+            "private_key": "-----BEGIN PRIVATE KEY-----\nBLAHBLAHBLAH\n-----END PRIVATE KEY-----\n",
+            "project_id": "test-project-id",
+            "universe_domain": "test-universe-domain",
+        });
+        let service_account = ServiceAccountBuilder::new(service_account_info_json)
+            .with_quota_project_id(test_quota_project)
+            .build_access_token_credentials()?;
+        let fmt = format!("{service_account:?}");
+        assert!(fmt.contains("ServiceAccountCredentials"));
+        assert!(fmt.contains(test_quota_project));
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
+    async fn get_user_account_access_token_credentials_from_builder() -> Result<()> {
+        let test_quota_project = "test-quota-project";
+        let authorized_user = serde_json::json!({
+            "client_id": "test-client-id",
+            "client_secret": "test-client-secret",
+            "refresh_token": "test-refresh-token",
+            "type": "authorized_user",
+        });
+        let user_account = UserAccountCredentialBuilder::new(authorized_user)
+            .with_quota_project_id(test_quota_project)
+            .build_access_token_credentials()?;
+        let fmt = format!("{user_account:?}");
+        assert!(fmt.contains("UserCredentials"), "{fmt:?}");
+        assert!(fmt.contains(test_quota_project));
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
+    async fn get_impersonated_access_token_credentials_from_builder() -> Result<()> {
+        let contents = json!({
+            "type": "impersonated_service_account",
+            "service_account_impersonation_url": "https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+            "source_credentials": {
+                "type": "authorized_user",
+                "client_id": "test-client-id",
+                "client_secret": "test-client-secret",
+                "refresh_token": "test-refresh-token"
+            }
+        });
+
+        let quota_project = "test-quota-project";
+
+        let ic = ImpersonatedBuilder::new(contents)
+            .with_quota_project_id(quota_project)
+            .build_access_token_credentials()
+            .unwrap();
+
+        let fmt = format!("{ic:?}");
+        assert!(fmt.contains("ImpersonatedServiceAccount"));
+        assert!(fmt.contains(quota_project));
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
     async fn get_user_account_credentials_from_builder() -> Result<()> {
         let test_quota_project = "test-quota-project";
         let authorized_user = serde_json::json!({
@@ -641,6 +766,7 @@ mod tests {
     }
 
     #[tokio::test]
+    #[parallel]
     async fn create_programmatic_token_provider_user_error() -> TestResult {
         let provider = FailingSubjectTokenProvider;
         let creds = ProgrammaticBuilder::new(std::sync::Arc::new(provider))
@@ -658,17 +784,8 @@ mod tests {
         assert!(original_error.to_string().contains("TestProviderError"));
         Ok(())
     }
-}
-
-#[cfg(all(test, google_cloud_unstable_id_token))]
-mod unstable_tests {
-    use super::tests::*;
-    use scoped_env::ScopedEnv;
-    use serde_json::json;
-    use serial_test::serial;
-
-    type TestResult = anyhow::Result<()>;
 
+    #[cfg(all(test, feature = "idtoken"))]
     #[tokio::test]
     #[serial]
     async fn create_id_token_credentials_fallback_to_mds() -> TestResult {
@@ -684,6 +801,7 @@ mod unstable_tests {
         Ok(())
     }
 
+    #[cfg(all(test, feature = "idtoken"))]
     #[tokio::test]
     #[serial]
     async fn create_id_token_credentials_adc_service_account() -> TestResult {
@@ -706,6 +824,7 @@ mod unstable_tests {
         Ok(())
     }
 
+    #[cfg(all(test, feature = "idtoken"))]
     #[tokio::test]
     #[serial]
     async fn create_id_token_credentials_adc_impersonated_service_account() -> TestResult {
@@ -732,6 +851,7 @@ mod unstable_tests {
         Ok(())
     }
 
+    #[cfg(all(test, feature = "idtoken"))]
     #[tokio::test]
     #[serial]
     async fn create_id_token_credentials_adc_user_account_not_supported() -> TestResult {
@@ -757,4 +877,92 @@ mod unstable_tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    #[serial]
+    async fn create_signer_credentials_fallback_to_mds() -> TestResult {
+        let _e1 = ScopedEnv::remove("GOOGLE_APPLICATION_CREDENTIALS");
+        let _e2 = ScopedEnv::remove("HOME"); // For posix
+        let _e3 = ScopedEnv::remove("APPDATA"); // For windows
+
+        let signer = google_cloud_auth::credentials::Builder::default().build_signer()?;
+        let fmt = format!("{signer:?}");
+        assert!(fmt.contains("MDSSigner"));
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[serial]
+    async fn create_signer_credentials_adc_service_account() -> TestResult {
+        let contents = r#"{
+            "type": "service_account",
+            "project_id": "test-project-id",
+            "private_key_id": "test-private-key-id",
+            "private_key": "-----BEGIN PRIVATE KEY-----\nBLAHBLAHBLAH\n-----END PRIVATE KEY-----\n",
+            "client_email": "test-client-email",
+            "universe_domain": "test-universe-domain"
+        }"#;
+        let path = write_cred_json(contents);
+        let _e = ScopedEnv::set("GOOGLE_APPLICATION_CREDENTIALS", path.to_str().unwrap());
+
+        let signer = google_cloud_auth::credentials::Builder::default().build_signer()?;
+        let fmt = format!("{signer:?}");
+        assert!(fmt.contains("ServiceAccountSigner"));
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[serial]
+    async fn create_signer_credentials_adc_impersonated_service_account() -> TestResult {
+        let contents = json!({
+            "type": "impersonated_service_account",
+            "service_account_impersonation_url": "https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/test-principal:generateAccessToken",
+            "source_credentials": {
+                "type": "authorized_user",
+                "client_id": "test-client-id",
+                "client_secret": "test-client-secret",
+                "refresh_token": "test-refresh-token"
+            }
+        })
+        .to_string();
+
+        let path = write_cred_json(&contents);
+        let _e = ScopedEnv::set("GOOGLE_APPLICATION_CREDENTIALS", path.to_str().unwrap());
+
+        let signer = google_cloud_auth::credentials::Builder::default().build_signer()?;
+        let fmt = format!("{signer:?}");
+        assert!(fmt.contains("IamSigner"));
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[serial]
+    async fn create_signer_credentials_adc_user_account_not_supported() -> TestResult {
+        let contents = r#"{
+                "client_id": "test-client-id",
+                "client_secret": "test-client-secret",
+                "refresh_token": "test-refresh-token",
+                "type": "authorized_user"
+            }"#;
+
+        let path = write_cred_json(contents);
+        let _e = ScopedEnv::set("GOOGLE_APPLICATION_CREDENTIALS", path.to_str().unwrap());
+
+        let err = google_cloud_auth::credentials::Builder::default()
+            .build_signer()
+            .unwrap_err();
+
+        assert!(err.is_not_supported());
+        assert!(
+            err.to_string()
+                .contains("authorized_user signer is not supported"),
+            "{}",
+            err
+        );
+
+        Ok(())
+    }
 }
