@@ -306,7 +306,7 @@ impl<F, Cr> ClientBuilder<F, Cr> {
     /// customize the default retry throtler.
     ///
     /// [Handling Overload]: https://sre.google/sre-book/handling-overload/
-    /// [Addressing Cascading Failures]: https://sre.google/sre-book/addressing-cascading-failures/
+    /// [Address Cascading Failures]: https://sre.google/sre-book/addressing-cascading-failures/
     ///
     /// ```
     /// # use google_cloud_gax::client_builder::examples;
@@ -423,6 +423,10 @@ pub mod internal {
         pub retry_throttler: SharedRetryThrottler,
         pub polling_error_policy: Option<Arc<dyn PollingErrorPolicy>>,
         pub polling_backoff_policy: Option<Arc<dyn PollingBackoffPolicy>>,
+        pub disable_automatic_decompression: bool,
+        pub disable_follow_redirects: bool,
+        pub grpc_subchannel_count: Option<usize>,
+        pub grpc_request_buffer_capacity: Option<usize>,
     }
 
     impl<Cr> std::default::Default for ClientConfig<Cr> {
@@ -438,9 +442,39 @@ pub mod internal {
                 retry_throttler: Arc::new(Mutex::new(AdaptiveThrottler::default())),
                 polling_error_policy: None,
                 polling_backoff_policy: None,
+                disable_automatic_decompression: false,
+                disable_follow_redirects: false,
+                grpc_subchannel_count: None,
+                grpc_request_buffer_capacity: None,
             }
         }
     }
+
+    /// Configure automatic decompression.
+    ///
+    /// By default, the client libraries automatically decompress responses.
+    /// Internal users can disable this behavior if they need to access the raw
+    /// compressed bytes.
+    pub fn with_automatic_decompression<F, Cr>(
+        mut builder: super::ClientBuilder<F, Cr>,
+        v: bool,
+    ) -> super::ClientBuilder<F, Cr> {
+        builder.config.disable_automatic_decompression = !v;
+        builder
+    }
+
+    /// Configure HTTP redirects.
+    ///
+    /// By default, the client libraries automatically follow HTTP redirects.
+    /// Internal users can disable this behavior if they need to handle redirects
+    /// manually (e.g. for 308 Resume Incomplete).
+    pub fn with_follow_redirects<F, Cr>(
+        mut builder: super::ClientBuilder<F, Cr>,
+        v: bool,
+    ) -> super::ClientBuilder<F, Cr> {
+        builder.config.disable_follow_redirects = !v;
+        builder
+    }
 }
 
 #[doc(hidden)]
@@ -531,6 +565,8 @@ pub mod examples {
             assert!(config.backoff_policy.is_none(), "{config:?}");
             assert!(config.polling_error_policy.is_none(), "{config:?}");
             assert!(config.polling_backoff_policy.is_none(), "{config:?}");
+            assert!(!config.disable_automatic_decompression, "{config:?}");
+            assert!(!config.disable_follow_redirects, "{config:?}");
         }
 
         #[tokio::test]
@@ -551,6 +587,44 @@ pub mod examples {
             assert!(config.tracing);
         }
 
+        #[tokio::test]
+        async fn automatic_decompression() {
+            let client = Client::builder();
+            let client = super::super::internal::with_automatic_decompression(client, false)
+                .build()
+                .await
+                .unwrap();
+            let config = client.0;
+            assert!(config.disable_automatic_decompression);
+
+            let client = Client::builder();
+            let client = super::super::internal::with_automatic_decompression(client, true)
+                .build()
+                .await
+                .unwrap();
+            let config = client.0;
+            assert!(!config.disable_automatic_decompression);
+        }
+
+        #[tokio::test]
+        async fn follow_redirects() {
+            let client = Client::builder();
+            let client = super::super::internal::with_follow_redirects(client, false)
+                .build()
+                .await
+                .unwrap();
+            let config = client.0;
+            assert!(config.disable_follow_redirects);
+
+            let client = Client::builder();
+            let client = super::super::internal::with_follow_redirects(client, true)
+                .build()
+                .await
+                .unwrap();
+            let config = client.0;
+            assert!(!config.disable_follow_redirects);
+        }
+
         #[tokio::test]
         async fn credentials() {
             let client = Client::builder()
