@@ -521,10 +521,9 @@ mod tests {
     }
 
     #[test]
-    fn aip194_strict() {
+    fn aip194_strict() -> anyhow::Result<()> {
         let p = Aip194Strict;
-
-        assert!(p.on_in_progress(&PollingState::default(), "unused").is_ok());
+        p.on_in_progress(&PollingState::default(), "unused")?;
         assert!(
             p.on_error(&PollingState::default(), unavailable())
                 .is_continue()
@@ -559,13 +558,15 @@ mod tests {
             p.on_error(&PollingState::default(), Error::ser("err".to_string()))
                 .is_permanent()
         );
+        Ok(())
     }
 
     #[test]
     fn always_continue() {
         let p = AlwaysContinue;
 
-        assert!(p.on_in_progress(&PollingState::default(), "unused").is_ok());
+        let result = p.on_in_progress(&PollingState::default(), "unused");
+        assert!(result.is_ok(), "{result:?}");
         assert!(
             p.on_error(&PollingState::default(), http_unavailable())
                 .is_continue()
@@ -713,7 +714,7 @@ mod tests {
             )
             .unwrap_err();
         let exhausted = err.source().and_then(|e| e.downcast_ref::<Exhausted>());
-        assert!(exhausted.is_some());
+        assert!(exhausted.is_some(), "{err:?}");
     }
 
     #[test]
@@ -896,7 +897,7 @@ mod tests {
             )
             .unwrap_err();
         let exhausted = err.source().and_then(|e| e.downcast_ref::<Exhausted>());
-        assert!(exhausted.is_some());
+        assert!(exhausted.is_some(), "{err:?}");
     }
 
     #[test]
