@@ -376,6 +376,7 @@ mod tests {
         {
             let path = entry.path();
 
+            // Skip conda-meta directory
             if path
                 .components()
                 .map(std::path::Component::as_os_str)
@@ -383,6 +384,10 @@ mod tests {
             {
                 continue;
             }
+            // Skip CACHEDIR.TAG (created by Prefix::create for backup exclusion)
+            if path.file_name() == Some(OsStr::new("CACHEDIR.TAG")) {
+                continue;
+            }
             if path.is_file() {
                 // strip off the base directory so we can compare to our hard-coded list
                 let rel = path
