@@ -20,16 +20,10 @@ pub(crate) struct PypiPackageDataModel<'a> {
     pub requires_dist: Cow<'a, [Requirement]>,
     #[serde(default, skip_serializing_if = "Option::is_none")]
     pub requires_python: Cow<'a, Option<VersionSpecifiers>>,
-    #[serde(default, skip_serializing_if = "should_skip_serializing_editable")]
+    #[serde(default)]
     pub editable: bool,
 }
 
-/// Used in `skip_serializing_if` to skip serializing the `editable` field if it
-/// is `false`.
-fn should_skip_serializing_editable(editable: &bool) -> bool {
-    !*editable
-}
-
 impl<'a> From<PypiPackageDataModel<'a>> for PypiPackageData {
     fn from(value: PypiPackageDataModel<'a>) -> Self {
         Self {
@@ -43,17 +37,3 @@ impl<'a> From<PypiPackageDataModel<'a>> for PypiPackageData {
         }
     }
 }
-
-impl<'a> From<&'a PypiPackageData> for PypiPackageDataModel<'a> {
-    fn from(value: &'a PypiPackageData) -> Self {
-        Self {
-            name: Cow::Borrowed(&value.name),
-            version: Cow::Borrowed(&value.version),
-            location: value.location.clone(),
-            hash: Cow::Borrowed(&value.hash),
-            requires_dist: Cow::Borrowed(&value.requires_dist),
-            requires_python: Cow::Borrowed(&value.requires_python),
-            editable: value.editable,
-        }
-    }
-}
