@@ -1,5 +1,5 @@
 use rattler_conda_types::compression_level::CompressionLevel;
-use rattler_conda_types::package::ArchiveType;
+use rattler_conda_types::package::CondaArchiveType;
 use rattler_package_streaming::read::{extract_conda_via_streaming, extract_tar_bz2};
 use rattler_package_streaming::write::{write_conda_package, write_tar_bz2_package};
 use std::collections::HashMap;
@@ -168,8 +168,8 @@ fn test_rewrite_tar_bz2() {
     let temp_dir = Path::new(env!("CARGO_TARGET_TMPDIR"));
     println!("Target dir: {}", temp_dir.display());
 
-    for file_path in
-        find_all_archives().filter(|path| ArchiveType::try_from(path) == Some(ArchiveType::TarBz2))
+    for file_path in find_all_archives()
+        .filter(|path| CondaArchiveType::try_from(path) == Some(CondaArchiveType::TarBz2))
     {
         println!("Name: {}", file_path.display());
 
@@ -207,7 +207,7 @@ fn test_rewrite_conda() {
     println!("Target dir: {}", temp_dir.display());
 
     for file_path in find_all_archives().filter(|path| {
-        ArchiveType::try_from(path) == Some(ArchiveType::Conda)
+        CondaArchiveType::try_from(path) == Some(CondaArchiveType::Conda)
             && path.file_name().unwrap() != "stir-5.0.2-py38h9224444_7.conda"
     }) {
         println!("Name: {}", file_path.display());
