@@ -81,7 +81,7 @@ pub async fn fetch_index(
         }
 
         // Decompress the bytes
-        let decoded_bytes = Bytes::from(decode_zst_bytes_async(bytes).await?);
+        let decoded_bytes = Bytes::from(decode_zst_bytes_async(bytes, response_url.clone()).await?);
 
         // The response is in, so we can drop the permit
         drop(permit);
