@@ -74,7 +74,7 @@ mod gateway;
 #[cfg(feature = "gateway")]
 pub use gateway::{
     CacheClearMode, ChannelConfig, Gateway, GatewayBuilder, GatewayError, MaxConcurrency, RepoData,
-    SourceConfig, SubdirSelection,
+    RepoDataSource, Source, SourceConfig, SubdirSelection,
 };
 #[cfg(feature = "indicatif")]
 pub use gateway::{IndicatifReporter, IndicatifReporterBuilder};
