@@ -1,6 +1,8 @@
 //! Defines several helper functions and structs that make it easier to
 //! implement a custom dependency provider.
 
+mod mapping;
 mod pool;
 
-pub use pool::{PackageName, Pool, VersionSet};
+pub use mapping::{Mapping, MappingIter};
+pub use pool::{PackageName, Pool, Solvable, VersionSet};
