@@ -11,10 +11,17 @@ use std::path::PathBuf;
 use std::time::SystemTime;
 
 pub mod boxed;
+pub mod ctx;
 pub mod impls;
 
 pub use sys_traits_macros::auto_impl;
 
+pub use self::ctx::FsFileWithPathsInErrors;
+pub use self::ctx::OperationError;
+pub use self::ctx::OperationErrorKind;
+pub use self::ctx::PathsInErrorsExt;
+pub use self::ctx::SysWithPathsInErrors;
+
 use self::boxed::BoxedFsFile;
 use self::boxed::BoxedFsMetadataValue;
 
@@ -791,10 +798,10 @@ impl<T: BaseFsRead> FsRead for T {}
 pub trait FsDirEntry: std::fmt::Debug {
   type Metadata: FsMetadataValue;
 
-  fn file_name(&self) -> Cow<OsStr>;
+  fn file_name(&self) -> Cow<'_, OsStr>;
   fn file_type(&self) -> io::Result<FileType>;
   fn metadata(&self) -> io::Result<Self::Metadata>;
-  fn path(&self) -> Cow<Path>;
+  fn path(&self) -> Cow<'_, Path>;
 }
 
 pub trait BaseFsReadDir {
